<?php

require_once 'WhereClause.php';
class SmartFormsQuery
{
    private $formId;
    public $selectColumns;
    public $fieldDictionary;

    /**
     * @var WhereClause[]
     */
    private $conditions;
    private $orderByColumns;

    public function __construct($formId)
    {
        $this->formId=$formId;
        $this->selectColumns=array();
        $this->conditions=array();
        $this->fieldDictionary=$this->GetFormDictionary();
    }

    public function StoresInformation($formelement)
    {
        switch($formelement["ClassName"])
        {
            case "rednaotextinput":
            case 'rednaorating':
            case "rednaoprependedtext":
            case "rednaoappendedtext":
            case "rednaoemail":
            case "rednaoprependedcheckbox":
            case "rednaoappendedcheckbox":
            case "rednaotextarea":
            case "rednaomultipleradios":
            case 'rednaomultiplecheckboxes ':
            case "rednaosearchablelist":
            case "rednaoimagepicker":
            case "rednaoselectbasic":
            case "rednaoname":
            case "rednaoaddress":
            case "rednaophone":
            case "rednaodonationrecurrence":
            case "sfFileUpload":
            case "rednaodatepicker":
            case "rednaosurveytable":
            case "rednaosignature":
            case "rednaorepeater":
            case "rednaonumber":
            case 'rednaocurrency':
            case 'rednaotimepicker':
                return true;

        }

        return false;
    }

    public function AddField($fieldId)
    {
        if(!isset($this->fieldDictionary[$fieldId]))
        {
            echo "Invalid field ".$fieldId;
            return;
        }
        array_push($this->selectColumns,$fieldId);
        return $this;
    }

    /**
     * @param $fieldList array
     */
    public function AddFields($fieldList){

        foreach($fieldList as $field)
            $this->AddField($field);
    }

    public function AddCondition($field,$comparison,$value,$join='and'){
        $whereCondition='';
        if(isset($this->fieldDictionary[$field]))
        {
            $whereCondition=new WhereClause($this->fieldDictionary[$field],$comparison,$value,$join);

        }else

        if($field=='_FormId'||$field=='_SFTimeStamp'||$field=='_UserId'||$field=='_Date')
        {
            $whereCondition=new WhereClause(array('ClassName'=>'','Id'=>$field),$comparison,$value,$join);
        }else{
            echo "Invalid field ".$field;
            return null;
        }

        $this->conditions[]=$whereCondition;
        return $whereCondition;


    }

    public function AddConditions($conditionList){
        for($i=0;$i<count($conditionList);$i++)
        {
            $whereClause=$this->AddCondition($conditionList[$i]['field'],$conditionList[$i]['comparison'],$conditionList[$i]['value']);
            if($i==0)
                $whereClause->openParentheses=true;
            if($i==count($conditionList)-1)
                $whereClause->closeParentheses=true;
            $this->conditions[]=$whereClause;
        }
    }

    public function GetResults($limit='',$skip=''){
        if(count($this->selectColumns)==0)
        {
            $this->selectColumns[]='_Date';
            foreach($this->fieldDictionary as $field)
            {
                if($this->StoresInformation($field))
                {
                    $this->selectColumns[]=$field['Id'];
                }
            }
            $this->selectColumns[]='_UserName';
        }
        return $this->Execute('',$limit,$skip);
    }

    public function GetRaw($limit='',$skip=''){
        return $this->Execute('parent.*',$limit,$skip);
    }

    public function GetCount(){
        $result=$this->Execute("count(*) count");
        if(count($result)==0)
            return null;
        return $result[0]['count'];
    }



    public function GetScalar(){
        $result=$this->Execute();
        if(count($result)==0)
            return null;
        return array_pop($result[0]);
    }

    private function Execute($columns='',$limit='',$skip=''){
        $useDefault=false;
        if($columns=='')
        {
            $useDefault=true;
            $columns='data,date,entry_id,form_id';
        }

        $db = JFactory::getDBO();
        $conditionText="SELECT $columns, user.name user_name,user.id user_id FROM #__smartforms_entry parent 
            left join #__users user on user.id=parent.user_id where form_id=".$db->quote($this->formId);


        if(count($this->conditions)>0)
        {

            foreach ($this->conditions as $condition)
            {
                $conditionText .=$condition->GetText();
            }
        }

        $conditionText.=' order by parent.date ';
        if($limit!=''&&$limit!=0)
        {
            $conditionText.=' limit '.intval($limit);
        }

        if($skip!='')
        {
            $conditionText.=' offset '.intval($skip);
        }

        $db->SetQuery($conditionText);

        $result=$db->loadAssocList();

        if($useDefault)
        {
            return $this->ProcessResult($result);
        }else{
            return $result;
        }



    }

    protected  function GetFormDictionary()
    {
        /** @noinspection PhpUndefinedMethodInspection */
        $db = JFactory::getDBO();
        $db->SetQuery("select element_options from #__smartforms where form_id=".$db->quote($this->formId));
        $result=$db->loadObjectList();
        if($result===false||count($result)==0)
            throw new Exception("Couldn't get form information");

        $formInfo=json_decode($result[0]->element_options,true);

        if($formInfo==null)
            throw new Exception("Couldn't get form information");

        /** @noinspection PhpIncludeInspection */
        include_once(SMART_FORMS_DIR.'string_renderer/rednao_string_builder.php');
        $StringBuilder=new rednao_string_builder();

        $formElementsDictionary =array();
        foreach($formInfo as $element)
        {
            $element["__renderer"]=$StringBuilder->GetElementRenderer($element);
            $formElementsDictionary[$element["Id"]]=$element;
        }

        return $formElementsDictionary;
    }

    private function ProcessResult($rows)
    {
        $result=array();
        foreach($rows as $row)
        {
            $entryData=array();

            $values = json_decode($row['data'],true);
            foreach ($this->selectColumns as $fieldId)
            {
                include_once(SMART_FORMS_DIR . 'string_renderer/rednao_string_builder.php');
                $stringBuilder = new rednao_string_builder();
                if($fieldId=='_Date')
                {
                    $entryData['_Date']=$row['date'];
                    continue;
                }

                if($fieldId=='_UserName')
                {
                    $entryData['_UserName']=$row['user_name'];
                    continue;
                }
                if(isset($values[$fieldId]))
                    $entryData[$fieldId] = $stringBuilder->GetStringFromColumn($this->fieldDictionary[$fieldId], $values[$fieldId]);
                else
                    $entryData[$fieldId]='';
            }
            $result[]=$entryData;
        }

        return $result;
    }


}


